'use strict';

const log4js = require('log4js');
const parseLib = require('./lib/parselib');
const execFuncLib = require('./lib/execfunclib');
const parser = require('./lib/grammer/javascript.pegjs');

const parsedMap = new Map();

const logger = log4js.getLogger();

// parse the mappingRulesStr (e.g.: format_dt('YYYY-MM-DD HH:mm:ss'); 
// return a statement Object which will be used for rule execution
// if failed to parse, return null
// INPUT: 
//  '"format_string('$name1' + '-' + '$name2');"
// OUTPUT:
//  statement Object
// {
//   funcName: the name of the function, e.g. 'format_dt, format_string, switch, ifBinary, ...'
//   params: the list of parameters which will the function will consume
// }
function parse(mappingRuleStr) {
    logger.debug('PARSE: ' + mappingRuleStr);
    let statement = null;

    // cache hit
    statement = parsedMap.get(mappingRuleStr);
    if (statement) {
        console.log(statement);
        return statement;
    }

    var cmd = parser.parse(mappingRuleStr);
    logger.debug('parsed: ' + JSON.stringify(cmd));

    // validate 
    if (cmd.type !== 'Program' || cmd.body[0] === 'undefined') {
        logger.error('failed to parse: ' + mappingRuleStr);
        return null;
    }

    var statementId = cmd.body[0].type;
    switch (statementId) {
    case 'SwitchStatement':
        statement = parseLib.parse_switch(mappingRuleStr);
        break;

    case 'IfStatement':
        var cmdBody = parser.parse(mappingRuleStr);
        if (cmdBody.body[0].test.type === 'BinaryExpression')
            statement = parseLib.parse_ifBinary(mappingRuleStr);
        else if (cmdBody.body[0].test.type === 'CallExpression')
            statement = parseLib.parse_ifFunction(mappingRuleStr);
        break;

    case 'ExpressionStatement':
        statement = parseLib.parse_func(mappingRuleStr);
        break;

    default:
        logger.error('unsupported rule :' + mappingRuleStr);
        break;
    }

    if (statement) {
        logger.debug('cache miss - added to cache' + mappingRuleStr);
        parsedMap.set(mappingRuleStr, statement);
    } else {
        logger.error('failed to parse: ' + mappingRuleStr);
    }

    return statement;
}

// Inputs: 
//  statement:
//  {
//     'name' : name of the function (located at execFuncLib)
//     'params'   : list of parameter passed into funcName(), params can be $nameN 
//  }
//  srcObj:  
//  {    
//      'name1':'val1', 
//      'name2':'val2',
//          ...,
//      'nameN':'valN'
//  }
// srcColumns: ['name1', 'name2']
//
// Output: 'val1-val2' (string)
// error return: null
function execFunction(statement, srcObj, srcColumns) {
    logger.debug('execFunction: ');
    let outputStr = null;
    const params = statement.params;

    switch (statement.name) {
    case 'format_dt':
        outputStr = execFuncLib.format_dt(srcObj[srcColumns[0]], params);
        break;

    case 'format_string':
        outputStr = execFuncLib.format_string(srcObj, srcColumns, params);
        break;

    case 'sum':
        outputStr = execFuncLib.sum(srcObj, srcColumns, params);
        break;

    case 'ifBinary':
        outputStr = execFuncLib.if_binary(srcObj, srcColumns, params);
        break;

    case 'switch':
        outputStr = execFuncLib.switch_case(srcObj, srcColumns, params);
        break;

    case 'format_dt1':
        outputStr = execFuncLib.format_dt1(srcObj, params);
        break;

    case 'ifFunction':
        outputStr = execFuncLib.if_function(srcObj, params);
        break;

    case 'format_invoice':
        outputStr = execFuncLib.format_invoice(srcObj, params);
        break;

    case 'format_amount':
        outputStr = execFuncLib.format_amount(srcObj, params);
        break;

    case 'getUpdateStmntForTierMapping':
        outputStr = execFuncLib.getUpdateStmntForTierMapping(srcObj, params);
        break;

    case 'select_record':
        outputStr = execFuncLib.select_record(srcObj, params);
        break;

    default:
        logger.warn('unsupported function: ' + statement.funcName);
        break;
    }

    // console.log('>'+outputStr+'<');
    return outputStr;
}

// helper function, this function takes source and recipe and calls parse and execution
// Inputs:
//  srcObj:  
//  {    
//      'name1':'val1', 
//      'name2':'val2',
//          ...,
//      'nameN':'valN'
//  }
//  recipeObj:
//  {
//      "source_fields": ["name1", "name2],
//      "mapping_rule": "format_dt('YYYY-MM-DD HH:mm:ss')";
//  }
// Output:
//    String: transformed value
// Error return: null
function transformOnRecipe(srcObj, recipeObj) {
    let transformedString = null;

    if (!recipeObj || !srcObj || !recipeObj.source_fields) {
        logger.error('transformOnReceipe: invalid srcObj or recipeObj');
        return null;
    }

    if (!Array.isArray(recipeObj.source_fields)) {
        logger.error('transformOnReceipe: recipe has wrong format, sources_fields is not array');
        return null;
    }

    // make sure fields specified in sources_fields are available in srcObj
    for (let i = 0; i < recipeObj.source_fields.length; i += 1) {
        if (!srcObj[recipeObj.source_fields[i]]) {
            logger.error('invalid Recipe : property ' + recipeObj.source_fields[i] + ' not found in srcObj');
            return null;
        }
    }

    // if mapping_rule is not exist or empty, return the specified source attribute
    if (!recipeObj.mapping_rule || (recipeObj.mapping_rule && recipeObj.mapping_rule.trim() === '')) {
        if (recipeObj.source_fields.length === 1) {
            logger.debug('transformOnRecipe: no mapping rule, return src');
            return srcObj[recipeObj.source_fields[0]];
        } else {
            logger.error('transformOnRecipe: no mapping rule, but source_fields contains more than 1 items');
            return null;
        }
    }

    // mapping_rule isn't empty'

    let statement = parse(recipeObj.mapping_rule);
    if (!statement) {
        logger.error('parsing error');
        return null;
    }
    transformedString = execFunction(statement, srcObj, recipeObj.source_fields);

    return transformedString;
}

module.exports = {
    parse,
    execFunction,
    transformOnRecipe
};