'use strict';
const rules = require('../index.js');
const expect = require('chai').expect;

const testSrc = {
    "varchar1" : "char1", 
    "varchar2" : "char2",
    "num1" : "1", 
    "num2" : "2",
    "datetime1" : "20160101000000",
    "datetime2" : "20160101235959", 
    "datetime3" : "20160101"
};


describe('format_dt', ()=>{
    const recipe = {
        "source_fields":["datetime1"],
        "mapping_rule" : "format_dt('YYYY-MM-DD HH:mm:ss');"
    };  

    it('format_dt(\'YYYY-MM-DD HH:mm:ss\') will format \'20160101000000\' to 2016-01-01 00:00:00', ()=>{
        const result = rules.transformOnRecipe(testSrc, recipe);
        expect(result).to.be.equal('2016-01-01 00:00:00');
    });
});

describe('format_dt_1st_day_next_month', ()=>{
    const recipe = {
        "source_fields":["datetime1"],
        "mapping_rule" : "format_dt('YYYY-MM-DD', 'months', 'Add', '1', 'startOf');"
    };
    it('format_dt(\'YYYY-MM-DD\', \'months\', \'Add\', \'1\', \'startOf\') will format \'20160101000000\' to 2016-02-01', ()=>{
        const result = rules.transformOnRecipe(testSrc, recipe);
        expect(result).to.be.equal('2016-02-01');
    });
});


describe('format_string', ()=>{
    const recipe = {
        "source_fields": ["num1", "num2"],
        "mapping_rule": "format_string($num1 + '...' + $num2);"
    };
    
    it('format_string($num1 +  \'...\' + $num2) will return to \'1..2\')', ()=>{
        const result = rules.transformOnRecipe(testSrc, recipe);
        expect(result).to.be.equal('1...2');
    });
});

describe('sum', ()=>{
    const recipe = {
        "source_fields": ["num1", "num2"],
        "mapping_rule": "sum($num1, $num2, 100);"
    }; 

    it('sum($num1, $num2, 100) will return to 103)', ()=>{
        const result = rules.transformOnRecipe(testSrc, recipe);
        expect(result).to.be.equal('103');
    });
});


describe('if matches, then (return value)', ()=>{
    const recipe = {
        "source_fields": ["varchar1", "varchar2"],
        "mapping_rule": "if($varchar1 == 'char1') $='success'; else $='fail';"
    }; 

    it('if($varchar1 == \'char1\') will return \'success\')', ()=>{
        const result = rules.transformOnRecipe(testSrc, recipe);
        console.log(result);
        expect(result).to.be.equal('success');
    });
});

describe('if matches, then (return variable)', ()=>{
    const recipe = {
        "source_fields": ["varchar1", "varchar2"],
        "mapping_rule": "if($varchar1 == 'char1') $=$varchar2; else $='fail';"
    }; 

    it('if($varchar1 == \'char1\') will return \'char2\')', ()=>{
        const result = rules.transformOnRecipe(testSrc, recipe);
        console.log(result);
        expect(result).to.be.equal('char2');
    });
});

describe('if NOT matches, then return else (value)', ()=>{
    const recipe = {
        "source_fields": ["varchar1", "varchar2"],
        "mapping_rule": "if($varchar1 != 'char1') $=$varchar2; else $='success';"
    }; 

    it('if($varchar1 != \'char1\') will return \'success\')', ()=>{
        const result = rules.transformOnRecipe(testSrc, recipe);
        expect(result).to.be.equal('success');
    });
});


describe('switch hit-case', ()=>{
    const recipe = {
        "source_fields": ["varchar1", "varchar2"],
        "mapping_rule": "switch($varchar1){case 'char1': $='1'; case 'char2': $=2; default: $='3';};"
    }; 

    it('switch case \'char1\' will return \'1\')', ()=>{
        const result = rules.transformOnRecipe(testSrc, recipe);
        expect(result).to.be.equal('1');
    });
});

describe('switch hit-default', ()=>{
    const recipe = {
        "source_fields": ["varchar1", "varchar2"],
        "mapping_rule": "switch($varchar1){case 'default': $='1'; case 'char2': $=2; default: $='3';};"
    }; 

    it('switch case \'char1\' will return \'3\')', ()=>{
        const result = rules.transformOnRecipe(testSrc, recipe);
        expect(result).to.be.equal('3');
    });
});